@@ -46,6 +46,10 @@ class DedentNode(Node): pass
 
 class TemplateNode(HierNode):
 
+    class TemplateTail(Node):
+        def py(self):
+            yield self.line('template = kajiki.Template(template)')
+
     def __init__(self, mod_py=None, defs=None):
         super(TemplateNode, self).__init__(defs)
         if mod_py is None: mod_py = []
@@ -53,7 +57,6 @@ class TemplateNode(HierNode):
         self.mod_py = [ x for x in mod_py if x is not None ]
 
     def py(self):
-        yield self.line('@kajiki.Template')
         yield self.line('class template:')
 
     def __iter__(self):
@@ -61,6 +64,7 @@ class TemplateNode(HierNode):
             yield x
         for x in super(TemplateNode, self).__iter__():
             yield x
+        yield self.TemplateTail()
 
 class ImportNode(Node):
 
@@ -145,6 +149,37 @@ class ForNode(HierNode):
     def py(self):
         yield self.line('for %s:' % (self.decl))
 
+class WithNode(HierNode):
+
+    class WithTail(Node):
+        def __init__(self, vars):
+            super(WithNode.WithTail, self).__init__()
+            self.vars = vars
+        def py(self):
+            gen = gen_name()
+            yield self.line('%s = local.__kj__.pop_with()' % gen)
+            for v in self.vars:
+                yield self.line('%s = %s.get(%r)' % (v, gen, v))
+                # yield self.line('if %s == (): del %s' % (v, v))
+
+    def __init__(self, vars, *body):
+        super(WithNode, self).__init__(body)
+        self.vars_text = vars
+        self.vars = dict(var.split('=')
+                         for var in vars.split(';'))
+
+    def py(self):
+        yield self.line(
+            'local.__kj__.push_with(locals(), %s)' % self.vars_text)
+        for k,v in self.vars.iteritems():
+            yield self.line('%s = %s' % (k,v))
+        
+    def __iter__(self):
+        yield self
+        for x in self.body_iter(): yield x
+        yield self.WithTail(self.vars)
+    
+
 class SwitchNode(HierNode):
 
     class SwitchTail(Node):
@@ -218,17 +253,21 @@ class TranslatableTextNode(TextNode):
 
 class ExprNode(Node):
 
-    def __init__(self, text):
+    def __init__(self, text, safe=False):
         super(ExprNode, self).__init__()
         self.text = text
+        self.safe = safe
 
     def py(self):
-        yield self.line('yield self.__kj__.escape(%s)' % self.text)
+        if self.safe:
+            yield self.line('yield %s' % self.text)
+        else:
+            yield self.line('yield self.__kj__.escape(%s)' % self.text)
 
 class PassNode(Node):
 
     def py(self):
-        yield self.line('pass')
+        yield self.line('yield ""')
 
 class AttrNode(HierNode):
 
@@ -239,7 +278,7 @@ class AttrNode(HierNode):
         def py(self):
             gen = self.p.genname
             x = gen_name()
-            yield self.line("%s = ''.join(%s())" % (gen, gen))
+            yield self.line("%s = self.__kj__.collect(%s())" % (gen, gen))
             yield self.line(
                 'for %s in self.__kj__.render_attrs({%r:%s}, %r):'
                 % (x, self.p.attr, gen, self.p.mode))
