@@ -52,8 +52,12 @@ class _Template(object):
             import_=self._import,
             escape=self._escape,
             gettext=i18n.gettext,
-            render_attrs=self._render_attrs)
+            render_attrs=self._render_attrs,
+            push_with=self._push_with,
+            pop_with=self._pop_with,
+            collect=self._collect)
         self._switch_stack = []
+        self._with_stack = []
         self.__globals__.update(context)
 
     def __iter__(self):
@@ -63,6 +67,14 @@ class _Template(object):
     def render(self):
         return u''.join(self)
 
+    def _push_with(self, lcls, **kw):
+        d = dict((k,lcls.get(k, ()))
+                 for k in kw)
+        self._with_stack.append(d)
+
+    def _pop_with(self):
+        return self._with_stack.pop()
+
     def _extend(self, parent):
         if isinstance(parent, basestring):
             parent = self.loader.import_(parent)
@@ -111,6 +123,9 @@ class _Template(object):
         return r
 
     def _escape(self, value):
+        if value is None: return value
+        if hasattr(value, '__html__'):
+            return value.__html__()
         if type(value) == flattener:
             return value
         uval = unicode(value)
@@ -122,10 +137,21 @@ class _Template(object):
     def _render_attrs(self, attrs, mode):
         if hasattr(attrs, 'items'):
             attrs = attrs.items()
-        for k,v in attrs:
-            if v is None: continue
-            if mode.startswith('html') and k in HTML_EMPTY_ATTRS: yield ' '+k.upper()
-            else: yield ' %s="%s"' % (k,self._escape(v))
+        if attrs is not None:        
+            for k,v in attrs:
+                if v is None: continue
+                if mode.startswith('html') and k in HTML_EMPTY_ATTRS: yield ' '+k.upper()
+                else: yield ' %s="%s"' % (k,self._escape(v))
+
+    def _collect(self, it):
+        result = []
+        for part in it:
+            if part is None: continue
+            result.append(part)
+        if result:
+            return u''.join(result)
+        else:
+            return None
 
     @classmethod
     def annotate_lnotab(cls, py_to_tpl):
